#include "qrwho.h"
#include "qprocess.h"
#include "qmessagebox.h"
#include <iostream>
#include <QFile>

QRwho::QRwho(QWidget *parent, Qt::WFlags flags)
    : QMainWindow(parent, flags)
{
    ui.setupUi(this);
    pclength = 0;
    row=1;
    column=1;
    columns=12;
    //CheckHosts();
    t = new Timer(this);
    QObject::connect(t, SIGNAL(howl()), this, SLOT(CheckHosts()));
    QObject::connect(t, SIGNAL(howl()), this, SLOT(SearchHost()));
    t->start();

    NodeCMD1= "lspci";
    NodeCMD2= "dmesg";
    NodeCMD3= "uname -a";
    NodeCMD4= "lsb_release -c";
    NodeCMD5= "ps auxf";

    min_red = 100;
    min_green = 255;
    min_blue = 255;
    max_red = 100;
    max_green = 200;
    max_blue = 0;

    cmds_on_off = 1;

    QFile config("config");
     if (!config.open(QIODevice::ReadOnly | QIODevice::Text))
         return;

     while (!config.atEnd()) {
         QByteArray line = config.readLine();
         process_config(line);
     }

     config.close();
}

QRwho::~QRwho()
{
 
}

void QRwho::process_config(QByteArray arg){
    QString line = QString(arg);
    if(QString("MaxLoad=") == line.left(8)){
        MaxLoad=line.right(line.length()-8).toFloat();
        std::cout << MaxLoad << "\n";
    }
    if(QString("CMDS=") == line.left(5)){
        cmds_on_off=line.right(line.length()-5).toFloat();
        std::cout << cmds_on_off << "\n";
    }
    if(QString("NodeCMD1=") == line.left(9)){
        NodeCMD1=line.right(line.length()-9);
        std::cout << NodeCMD1.toStdString();
    }
    if(QString("NodeCMD2=") ==(line.left(9))){
        NodeCMD2=line.right(line.length()-9);
        std::cout << NodeCMD2.toStdString();
    }
    if(QString("NodeCMD3=") ==(line.left(9))){
        NodeCMD3=line.right(line.length()-9);
        std::cout << NodeCMD3.toStdString();
    }
    if(QString("NodeCMD4=") ==(line.left(9))){
        NodeCMD4=line.right(line.length()-9);
        std::cout << NodeCMD4.toStdString();
    }
    if(QString("NodeCMD5=") ==(line.left(9))){
        NodeCMD5=line.right(line.length()-9);
        std::cout << NodeCMD5.toStdString();
    }
    if(QString("MinColor=") ==(line.left(9))){
        min_red = line.mid(9,line.indexOf(",")-9).toInt(false,10);
        min_green = line.mid(line.indexOf(",")+1,line.indexOf(",",line.indexOf(",")+1)-line.indexOf(",")-1).toInt(false,10);
        min_blue = line.mid(line.indexOf(",",line.indexOf(",")+1)+1,line.length()-line.indexOf(",",line.indexOf(",")+1)).toInt(false,10);
        std::cout << min_red << "\n";
        std::cout << min_green << "\n";
        std::cout << min_blue << "\n";
    }
    if(QString("MaxColor=") ==(line.left(9))){
        max_red = line.mid(9,line.indexOf(",")-9).toInt(false,10);
        max_green = line.mid(line.indexOf(",")+1,line.indexOf(",",line.indexOf(",")+1)-line.indexOf(",")-1).toInt(false,10);
        max_blue = line.mid(line.indexOf(",",line.indexOf(",")+1)+1,line.length()-line.indexOf(",",line.indexOf(",")+1)).toInt(false,10);
        std::cout << max_red << "\n";
        std::cout << max_green << "\n";
        std::cout << max_blue << "\n";
    }
}
void QRwho::resizeEvent(QResizeEvent *e){
    SearchHost();
}

QString QRwho::stripWhiteSpaces(QString str){
    while(str.indexOf(QString("  ")) != -1){
        str.replace(QString("  "),QString(" "));
    }
    return str;
}

int QRwho::indexOfPc(QString name){
    int a =-1;
    for(int i=0;i<pclength;i++){
        if (pc[i]==name)
            a=i;
    }
    return a;
}

void QRwho::removeHosts(){
    for (int i=0;i<pclength;i++) {
        ui.gridLayout->removeWidget(pcs[i]);
        pcs[i]->hide();
    }
    column=1;
    row=1;
}

void QRwho::removeUser(){
    for (int i=0;i<pclength;i++) {
        pcs[i]->clearUser();
    }
}

void QRwho::SearchHost(){
    removeHosts();
    columns = ui.centralWidget->width() / 120;
    //std::cout << columns;
    for (int i=0;i<pclength;i++) {
        if(pc[i].indexOf(ui.lineEdit->text()) != -1){
            if (column < columns) {
                ui.gridLayout->addWidget(pcs[i],row,column);
                column+=1;
            }else{
                ui.gridLayout->addWidget(pcs[i],row,column);
                column=1;
                row+=1;
            }
            pcs[i]->show();
        }else{
            if(pcs[i]->indexOfUser(ui.lineEdit->text()) != -1){
                if (column < columns) {
                    ui.gridLayout->addWidget(pcs[i],row,column);
                    column+=1;
                }else{
                    ui.gridLayout->addWidget(pcs[i],row,column);
                    column=1;
                    row+=1;
                }
                pcs[i]->show();
            }
        }

    }
}

void QRwho::setHostFocus(QString host){
    if (indexOfPc(host) != -1)
        pcs[indexOfPc(host)]->setFocus();
}

int QRwho::indexOfCount(QString str, QString substr, int n){
    int i = 0;
    int count = 0;
    int index = -1;
    while(count < n && i < str.count()){
        //QMessageBox::about(this,QString("test"),str.mid(i,substr.count()));
        if(str.mid(i,substr.count()) == substr){
            index = i;
            count += 1;
        }
       i += 1;
    }
    return index;
}

void QRwho::addHost(QString name){
    pcs[pclength] = new XRwhoPC(this,NodeCMD1,NodeCMD2,NodeCMD3,NodeCMD4,NodeCMD5);
    pcs[pclength]->setPCName(name);
    pcs[pclength]->show();
    pcs[pclength]->clearUser();
    if(!cmds_on_off){
        pcs[pclength]->hideCMDS();
    }else{
        pcs[pclength]->showCMDS();
    }
    if (column < columns) {
        ui.gridLayout->addWidget(pcs[pclength],row,column);
        column+=1;
    }else{
        ui.gridLayout->addWidget(pcs[pclength],row,column);
        column=1;
        row+=1;
    }
    pc[pclength]= name;
    pclength += 1;
}

void QRwho::addUser(QString name, int n){
    pcs[n]->addUser(name);
}

void QRwho::addLoad(QString load, int n){
    float f_red, f_green, f_blue;
    pcs[n]->setLoad(load);
    pcs[n]->hideDead();
    if(pcs[n]->max_load == 0){
        pcs[n]->max_load = getMaxLoad(n);
    }
    if(load == QString("down")){
        pcs[n]->showDead();
    }
    if(load.toFloat() > pcs[n]->max_load || load == QString("down")){
        pcs[n]->alert();
    }else{
        f_red = (max_red-min_red) / pcs[n]->max_load;
        f_green = (max_green-min_green) / pcs[n]->max_load;
        f_blue = (max_blue-min_blue) / pcs[n]->max_load;
        pcs[n]->normal(QColor(min_red + f_red*load.toFloat(),min_green + f_green*load.toFloat(),min_blue + f_blue*load.toFloat()));
    }
}

float QRwho::getMaxLoad(int n){
    QString name = pcs[n]->getPCName();
    QFile config("config");
     if (!config.open(QIODevice::ReadOnly | QIODevice::Text))
         return MaxLoad;

     while (!config.atEnd()) {
         QByteArray line = config.readLine();
         if(QString(line).left(3+name.length()) == QString(QString("ML")+name+QString("="))){
             std::cout << QString(line).right(QString(line).length() - 3 - name.length()).toFloat() << "\n";
             return QString(line).right(QString(line).length() - 3 - name.length()).toFloat();
         }
     }

     config.close();

     return MaxLoad;
}

QStringList QRwho::ListFromString(QString str){
    QStringList list;
    list = str.split("\n");
    return list;
}

void QRwho::CheckHosts(){
    //sleep(10);
    QString output;
    QStringList pclist;
    QString line;
    QString pcname;
    bool wait = true;
    QProcess *rwho = new QProcess(this);
    QProcess *ruptime = new QProcess(this);

    removeUser();

    rwho->start(QString("rwho -a"));
    //rwho->start(QString("cat /home/pneuser/test"));
    while(wait){
        if(!rwho->waitForFinished()){
            wait=false;
        }
    }
    output = QString(rwho->readAllStandardOutput());
    output = stripWhiteSpaces(output);
    //QMessageBox::about(this,QString("test"),output);
    pclist = ListFromString(output);
    for (int i=0;i<pclist.size()-1;i++){
        line = pclist[i];
        pcname = line.mid(line.indexOf(QString(" "))+1,(line.indexOf(QString(":"))-1-line.indexOf(QString(" "))));
        if(indexOfPc(pcname)== -1 ){
            addHost(pcname);
            addUser(line.left(line.indexOf(QString(" "))),indexOfPc(pcname));
        }else{
            addUser(line.left(line.indexOf(QString(" "))),indexOfPc(pcname));
        }
    }

    wait=true;
    ruptime->start(QString("ruptime"));
    //ruptime->start(QString("cat /home/pneuser/test2"));
    while(wait){
        if(!ruptime->waitForFinished()){
            wait=false;
        }
    }

    output = QString(ruptime->readAllStandardOutput());
    output = stripWhiteSpaces(output);
    //QMessageBox::about(this,QString("test"),output);
    pclist = ListFromString(output);
    for (int i=0;i<pclist.size()-1;i++){
        line = pclist[i];
        pcname = line.left(line.indexOf(QString(" ")));
        //QMessageBox::about(this,QString("test"),QString::number(indexOfCount(line,QString(" "),9)));
        if(indexOfPc(pcname)== -1 ){
            addHost(pcname);
            if(line.indexOf(" down ") == -1){
                //std::cout << "up\n";
                addLoad(line.mid(indexOfCount(line,QString(" "),9),indexOfCount(line,QString(","),3)-indexOfCount(line,QString(" "),9)),indexOfPc(pcname));
            }else{
                //std::cout << "down\n";
                addLoad(QString("down"),indexOfPc(pcname));
            }
        }else{
            if(line.indexOf(" down ") == -1){
                //std::cout << "up\n";
                addLoad(line.mid(indexOfCount(line,QString(" "),9),indexOfCount(line,QString(","),3)-indexOfCount(line,QString(" "),9)),indexOfPc(pcname));
            }else{
                //std::cout << "down\n";
                addLoad(QString("down"),indexOfPc(pcname));
            }
        }
    }
}


// timer functions

Timer::Timer(QObject *parent)
    : QThread(parent)
{
    offset = 180;
}

Timer::~Timer()
{

}

void Timer::run(){
    while(true){
        sleep(offset);
        emit howl();
    }
}
